#include "DataType.h"
#include <iterator>
#include <optional>

using oint = std::optional<int>;
oint ReadInt(std::istream& in) {
	int i;
	if (in >> i) 
		return { i }; 
	return {};
}



Node::Node(int node_id, const std::array<double, 3>& coord, bool vertex)
	: m_node_id(node_id)
	, m_coord(coord)
	, m_vertex(vertex)
{}

Node::Node(const Node& node)
	: m_node_id(node.m_node_id)
	, m_coord(node.m_coord)
	, m_vertex(node.m_vertex)
{}

std::istream& operator >> (std::istream& in, Node& node)
{
	in >> node.m_node_id;
	in >> get<0>(node.m_coord)
		>> get<1>(node.m_coord)
		>> get<2>(node.m_coord);
	in >> node.m_vertex;
	return in;
}

std::ostream& operator << (std::ostream& out, const Node& node)
{
	out << node.m_node_id;
	out << get<0>(node.m_coord)
		<< get<1>(node.m_coord)
		<< get<2>(node.m_coord);
	out << node.m_vertex;
	return out;
}



Edge::Edge(int id_first, int id_second)
	:m_node(std::make_pair(id_first, id_second))
{}

Edge  Edge::operator=(const Edge& edge)
{
	Edge temp;
	temp.m_node = edge.m_node;
	return temp;
}

bool Edge::operator==(const Edge& edge) const
{
	return(m_node.first == edge.m_node.first)
		&& (m_node.second == edge.m_node.second);
}

Edge Edge::create_edge(int first_id, int second_id)
{
	return Edge(first_id, second_id);
}

void Edge::update_middle(int middle_node)
{
	m_middle_node = middle_node;
}

std::istream& operator >> (std::istream& in, Edge& edge)
{
	in >> edge.m_node.first >> edge.m_node.second;
	return in;
}

std::ostream& operator << (std::ostream& out, const Edge& edge)
{
	out << "[ " << edge.m_node.first << " , " << edge.m_node.second << " ]";
	return out;
}



std::istream& operator >> (std::istream& in, FiniteElement& fin_el)
{
	in >> fin_el.m_element_id
		>> fin_el.m_material_id;
	oint node_id;
	node_id = ReadInt(in);
	while (node_id) {
		fin_el.m_node_id.push_back(*node_id);
		node_id = ReadInt(in);
	}
	return in;
}

std::ostream& operator<< (std::ostream& out, const FiniteElement& fin_el)
{
	out << fin_el.m_element_id << std::endl
		<< fin_el.m_material_id << std::endl;
	out << "[ ";
	for (const auto& it_elem : fin_el.m_node_id)
		out << it_elem << " ";
	out << " ]";
	return out;
}

FiniteElement::FiniteElement(int element_id, int material_id, const std::vector<int>& node_id)
	: m_element_id(element_id)
	, m_material_id(material_id)
	, m_node_id(node_id)
{}



BoundaryFiniteElement::BoundaryFiniteElement(int element_id, int surface_id, const std::vector<int>& node_id)
	: m_element_id(element_id)
	, m_surface_id(surface_id)
	, m_node_id(node_id)
{}

std::istream& operator >> (std::istream& in, BoundaryFiniteElement& bound_fin_el)
{
	in >> bound_fin_el.m_surface_id
		>> bound_fin_el.m_element_id;
	oint node_id;
	node_id = ReadInt(in);
	while (node_id) {
		bound_fin_el.m_node_id.push_back(*node_id);
		node_id = ReadInt(in);
	}
	return in;
}

std::ostream& operator << (std::ostream& out, const BoundaryFiniteElement& bound_fin_el)
{
	out << bound_fin_el.m_surface_id << std::endl
		<< bound_fin_el.m_element_id << std::endl;
	out << "[ ";
	for (const auto& it_elem : bound_fin_el.m_node_id)
		out << it_elem << " ";
	out << " ]";
	return out;
}