#include "aneu_MeshLoader.h"
#include "FileNotFound.h"
#include <iostream>
#include <fstream>

void aneu_MeshLoader::LoadMesh(const std::string& filename)
{
	std::ifstream fin(filename);
	if (!fin) {
		throw FileNotFound();
	}
	int number_of_node, dimesion, number_of_elem, node_in_elem, number_of_bound, node_in_bound;
	
	fin >> number_of_node >> dimesion;
	m_node.reserve(number_of_node);
	for (int i = 0; i < number_of_node; ++i) {
		Node a_node;
		a_node.m_node_id = i + 1;
		fin >> get<0>(a_node.m_coord) >> get<1>(a_node.m_coord) >> get<2>(a_node.m_coord);
		a_node.m_vertex = false;
		m_node.push_back(std::move(a_node));
	}
	
	fin >> number_of_elem >> node_in_elem;
	m_fin_el.reserve(number_of_elem);
	for (int i = 0; i < number_of_elem; ++i) {
		FiniteElement a_elem;
		a_elem.m_element_id = i + 1;
		fin >> a_elem.m_material_id;
		a_elem.m_node_id.reserve(node_in_elem);
		for (int j = 0; j < node_in_elem; ++j) {
			int node_id;
			fin >> node_id;
			a_elem.m_node_id.push_back(node_id);
		}
		m_fin_el.push_back(std::move(a_elem));
	}
	
	fin >> number_of_bound >> node_in_bound;
	m_bound_fin_el.reserve(number_of_bound);
	for (int i = 0; i < number_of_bound; ++i) {
		BoundaryFiniteElement a_bound;
		a_bound.m_element_id = i + 1;
		fin >> a_bound.m_surface_id;
		a_bound.m_node_id.reserve(node_in_bound);
		for (int j = 0; j < node_in_bound; ++j) {
			int node_id;
			fin >> node_id;
			m_node[node_id - 1].m_vertex = true;
			a_bound.m_node_id.push_back(node_id);
		}
		m_bound_fin_el.push_back(std::move(a_bound));
	}
}
